﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Steema.TeeChart.Styles;
using Steema.TeeChart;
using System.Threading;

namespace teechartScaleOverlapping
{
    public partial class FormReSize : Form
    {
        Steema.TeeChart.Styles.Line line1;
        Steema.TeeChart.TChart tChart1;
        public FormReSize()
        {
            InitializeComponent();
            tChart1 = new Steema.TeeChart.TChart();
            this.panel1.Controls.Add(tChart1);
            tChart1.Dock = DockStyle.Fill;
            ReDrawChart();

            tChart1.Resize += new EventHandler(tChart1_Resize);
            tChart1.Axes.Left.Labels.ValueFormat = "0.000E+00";
        }

        void tChart1_Resize(object sender, EventArgs e)
        {
         
            AdjustLabel();
            this.label1.Text += string.Format("tChart1_SizeChanged:Width:{0},Height:{1}{2}", tChart1.Width, tChart1.Height, Environment.NewLine);
        }

        private void AdjustLabel()
        {
            //the current Label  will be hidden when it is overlapped with its previous labels.
            //Get the axis
            Steema.TeeChart.Axis xAxis = tChart1.Axes.Bottom;
            Steema.TeeChart.Axis yAxis = tChart1.Axes.Left;
            xAxis.Labels.Items[0].Visible = true;
            string visibleInfo =  xAxis.Labels.Items[0].Text+":true"+Environment.NewLine;
            for (int i = 1; i < xAxis.Labels.Items.Count; i++)
            {
                var currentLabel = xAxis.Labels.Items[i];
                SizeF currentSize = tChart1.Graphics3D.MeasureString(xAxis.Labels.Font, currentLabel.Text);
                Rectangle currentRect = new Rectangle(xAxis.CalcXPosValue(currentLabel.Value), xAxis.CalcYPosValue(currentLabel.Value), (int)currentSize.Width, (int)currentSize.Height);
                AxisLabelItem lastVisibleLabel = xAxis.Labels.Items.Where(x => x.Visible && IndexOfLabel(xAxis, x.Text) < i).LastOrDefault();//get the nearest visible Label before current Label
                if (lastVisibleLabel == null)
                {
                    currentLabel.Visible = true;
                    continue;
                }
                SizeF lastVisibleSize = tChart1.Graphics3D.MeasureString(xAxis.Labels.Font, lastVisibleLabel.Text);
                Rectangle lasyVisibleRect = new Rectangle(xAxis.CalcXPosValue(lastVisibleLabel.Value), xAxis.CalcYPosValue(lastVisibleLabel.Value), (int)lastVisibleSize.Width, (int)lastVisibleSize.Height);
                if (lasyVisibleRect.X + lastVisibleLabel.Width >= currentRect.X)
                {
                    currentLabel.Visible = false;
                }
                else
                {
                    currentLabel.Visible = true;
                }
                visibleInfo += xAxis.Labels.Items[i].Text + currentLabel.Visible + Environment.NewLine;
            }
            this.label1.Text = visibleInfo;

           // tChart1.AfterDraw -= new PaintChartEventHandler(tChart1_AfterDraw);
            tChart1.Update();
           // tChart1.AfterDraw += new PaintChartEventHandler(tChart1_AfterDraw);
           
        }

        private int IndexOfLabel(Axis axis, string p)
        {
            var labelArray = axis.Labels.Items;

            for (int i = 0; i < labelArray.Count; i++)
            {
                if (labelArray[i].Text == p)
                {
                    return i;
                }
            }
            return -1;
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            ReDrawChart();
        }
        private void ReDrawChart()
        {
            tChart1.Series.Clear();
            Random rnd = new Random();
            line1 = new Line(tChart1.Chart);
            if (checkBox1.Checked)
            {
                line1.Add(1.00, rnd.Next(100));
                line1.Add(10.00, rnd.Next(100));
                line1.Add(100.00, rnd.Next(100));
                line1.Add(1000.00, rnd.Next(100));
                line1.Add(10000.00, rnd.Next(100));
                line1.Add(100000.00, rnd.Next(100));
                line1.Add(1000000.00, rnd.Next(100));
                line1.Add(10000000.00, rnd.Next(100));
                line1.Add(100000000.00, rnd.Next(100));
                line1.Add(1000000000.00, rnd.Next(100));

                tChart1.Axes.Bottom.Logarithmic = true;
                tChart1.Axes.Bottom.Labels.Items.Clear();

                for (int i = 1; i < 1000000000; i = i * 10)
                {
                    double x = i;
                    AxisLabelItem xLabel = tChart1.Axes.Bottom.Labels.Items.Add(Convert.ToDouble(x.ToString("E3")));
                    xLabel.Text = x.ToString("E3");
                    xLabel.Width = 100;
                    xLabel.Value = x;
                }
            }
            else
            {
                line1.Add(11, rnd.Next(100));
                line1.Add(21, rnd.Next(100));
                line1.Add(31, rnd.Next(100));
                line1.Add(41, rnd.Next(100));
                line1.Add(51, rnd.Next(100));
                line1.Add(61, rnd.Next(100));
                line1.Add(71, rnd.Next(100));
                line1.Add(81, rnd.Next(100));
                line1.Add(91, rnd.Next(100));
                line1.Add(101, rnd.Next(100));

                tChart1.Axes.Bottom.Logarithmic = false;
                tChart1.Axes.Bottom.Labels.Items.Clear();

                for (int i = 11; i <= 101; i = i + 10)
                {
                    double x = i;
                    AxisLabelItem xLabel = tChart1.Axes.Bottom.Labels.Items.Add(Convert.ToDouble(x.ToString("E3")));
                    xLabel.Text = x.ToString("E3");
                    xLabel.Value = x;
                }
            }
            this.label1.Text += string.Format("ReDrawChart:Width:{0},Height:{1}{2}", tChart1.Width, tChart1.Height, Environment.NewLine);
        }

        private void button1_Click(object sender, EventArgs e)
        {
            this.label1.Text = "";
        }

      

    }
}
